//	VDShader - Custom shader video filter for VirtualDub
//	Copyright (C) 2007 Avery Lee
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "stdafx.h"
#include "effect.h"

///////////////////////////////////////////////////////////////////////////////
namespace {
	unsigned hash(const char *s, size_t len) {
		unsigned h = 0;

		while(len--) {
			h = (h * 33) ^ (unsigned)*s++;
		}

		return h;
	}
}

bool VDShaderConstant::operator==(const VDShaderConstant& c) const {
	return v[0] == c.v[0]
		&& v[1] == c.v[1]
		&& v[2] == c.v[2]
		&& v[3] == c.v[3];
}

size_t VDShaderConstantHash::operator()(const VDShaderConstant& c) const {
	union { float f; int i; } k[4]={
		c.v[0], c.v[1], c.v[2], c.v[3],
	};

	return k[0].i + k[1].i + k[2].i + k[3].i;
}

VDShaderNameTable::VDShaderNameTable() {
	mNameHeap.push_back(0);
}

VDShaderNameTable::~VDShaderNameTable() {
}

int VDShaderNameTable::GetName(const char *s, size_t len) const {
	unsigned h = hash(s, len);

	std::pair<NameMap::const_iterator, NameMap::const_iterator> range(mNameMap.equal_range(h));

	for(; range.first != range.second; ++range.first) {
		int offset = range.first->second;

		const char *t = &mNameHeap[offset];
		size_t len2 = strlen(t);
		if (len == len2 && !memcmp(s, t, len))
			return offset;
	}

	return 0;
}

int VDShaderNameTable::AddName(const char *s, size_t len) {
	unsigned h = hash(s, len);

	std::pair<NameMap::const_iterator, NameMap::const_iterator> range(mNameMap.equal_range(h));

	for(; range.first != range.second; ++range.first) {
		int offset = range.first->second;

		const char *t = &mNameHeap[offset];
		size_t len2 = strlen(t);
		if (len == len2 && !memcmp(s, t, len))
			return offset;
	}

	int pos = (int)mNameHeap.size();

	mNameHeap.insert(mNameHeap.end(), s, s+len);
	mNameHeap.push_back(0);
	mNameMap.insert(NameMap::value_type(h, pos));
	return pos;
}

///////////////////////////////////////////////////////////////////////////////
VDShaderPass::VDShaderPass() {
}

VDShaderPass::~VDShaderPass() {
}

///////////////////////////////////////////////////////////////////////////////
VDShaderTechnique::VDShaderTechnique() {
}

VDShaderTechnique::~VDShaderTechnique() {
	while(!mPasses.empty()) {
		delete mPasses.back();
		mPasses.pop_back();
	}
}

///////////////////////////////////////////////////////////////////////////////
VDShaderEffect::VDShaderEffect() {
}

VDShaderEffect::~VDShaderEffect() {
	while(!mFunctions.empty()) {
		delete mFunctions.back();
		mFunctions.pop_back();
	}

	while(!mTechniques.empty()) {
		delete mTechniques.back();
		mTechniques.pop_back();
	}
}
